<?php
/**
 * DZF
 *
 * LICENSE: Redistribution and use in source and binary forms, with or without
 * modifications, are not permitted.
 *
 * @copyright Deubert IT-Solutions © 2017 All rights reserved
 * @package DZF
 */

#namespace DeubertIt;

/**
 * @author Eugene Melnik <e.melnik@luckywarepro.com>
 */
class DZF_Settings_Source_Doctrine extends DZF_Settings_Source
{
    /**
     * @var string
     */
    protected $_tableName;

    /**
     * @var string
     */
    protected $_groupKeyColumnName;

    /**
     * @var string
     */
    protected $_paramKeyColumnName;

    /**
     * @var string
     */
    protected $_valueColumnName;

    /**
     * @param   string      $tableName              (optional)
     * @param   string      $groupKeyColumnName     (optional)
     * @param   string      $paramKeyColumnName     (optional)
     * @param   string      $valueColumnName        (optional)
     */
    public function __construct($tableName = null, $groupKeyColumnName = null, $paramKeyColumnName = null, $valueColumnName = null)
    {
        $this->_tableName = is_null($tableName)
            ? DZF_Settings_Source_Doctrine_Model::SETTINGS_TABLE_NAME
            : $tableName;

        $this->_groupKeyColumnName = is_null($groupKeyColumnName)
            ? 'group_key'
            : $groupKeyColumnName;

        $this->_paramKeyColumnName = is_null($paramKeyColumnName)
            ? 'param_key'
            : $paramKeyColumnName;

        $this->_valueColumnName = is_null($valueColumnName)
            ? 'value'
            : $valueColumnName;
    }

    /**
     * @param   string          $groupKey
     * @param   string          $paramName      (optional)
     * @return  mixed|null
     */
    public function getValue($groupKey, $paramName = null)
    {
        $query = <<<QUERY_TEXT
SELECT `$this->_valueColumnName`
    FROM `$this->_tableName`
    WHERE `$this->_groupKeyColumnName` = :groupKey
QUERY_TEXT;

        $params = array(
            ':groupKey' => (string) $groupKey
        );

        if (is_null($paramName))
        {
            $query .= " AND `$this->_paramKeyColumnName` IS NULL";
        }
        else
        {
            $query .= " AND `$this->_paramKeyColumnName` = :paramName";
            $params[':paramName'] = (string) $paramName;
        }

        $statement = \Doctrine_Manager::getInstance()->connection()->prepare($query);

        foreach ($params as $paramName => $paramValue)
        {
            $statement->bindValue($paramName, $paramValue);
        }

        if ($statement->execute() && $statement->rowCount() > 0
            && ($result = $statement->fetchColumn()) !== false)
        {
            if (!empty($result))
            {
                try
                {
                    $result = \Zend_Json::decode($result);
                }
                catch (\Exception $e)
                {
                    $result = null;
                }
            }

            return $result;
        }

        return null;
    }
}
